/*
 * Copyright (c) 2015 embedded brains GmbH
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _LINUX_ERR_H
#define	_LINUX_ERR_H

#include <linux/compiler.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define IS_ERR_VALUE(e) unlikely((e) >= (unsigned long)-4095)

static inline void *
__must_check ERR_PTR(long e)
{

	return ((void *)e);
}

static inline long __must_check
PTR_ERR(__force const void *p)
{

	return ((long)p);
}

static inline bool __must_check
IS_ERR(__force const void *p)
{

	return (IS_ERR_VALUE((unsigned long)p));
}

static inline bool __must_check
IS_ERR_OR_NULL(__force const void *p)
{

	return (p == NULL || IS_ERR_VALUE((unsigned long)p));
}

static inline void * __must_check
ERR_CAST(__force const void *p)
{

	return (__DECONST(void *, p));
}

static inline int __must_check
PTR_ERR_OR_ZERO(__force const void *p)
{

	return (IS_ERR(p) ? (int)PTR_ERR(p) : 0);
}

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _LINUX_ERR_H */
